import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  ProtocolDecoder,
  ProtocolMessage,
  PacketBodyValue,
  BodyStringDivider,
  ProtocolStream
} from '@ucap-webmessenger/protocol';
import { EmployeeType } from '@ucap-webmessenger/protocol-room';
import { RoleCode } from '@ucap-webmessenger/protocol-authentication';
import { DeptSearchType } from '../types/dept-search.type';
import { CallMode } from '@ucap-webmessenger/core';

export interface DeptUserRequest extends ProtocolRequest {
  // DivCD(s)
  divCd: string;
  // 기관코드(s)
  companyCode: string;
  // 부서SEQ(n)
  seq?: number;
  // 검색string(s)
  search?: string;
  // 검색범위(s)
  searchRange: DeptSearchType;
  // 요청리스트(n) 사용안하면 0 최대 200
  pageListCount?: number;
  // 요청페이지(n) 사용안하면 0 처음은 1
  pageCurrent?: number;
  // 발신자회사코드(s)
  senderCompanyCode: string;
  // 발신자임직원유형(s)
  senderEmployeeType: EmployeeType;
}

export interface UserInfoSS {
  // 사용자SEQ
  seq: number;
  // 사용자명
  name: string;
  // 사진파일
  profileImageFile: string;
  // 직급
  grade: string;
  // 업무소개
  intro: string;
  // 기관코드
  companyCode: string;
  // 핸드폰번호
  hpNumber: string;
  // 내선번호
  lineNumber: string;
  // 이메일
  email: string;
  // 모바일YN
  isMobile: boolean;
  // 부서명
  deptName: string;
  // 조회순서
  order: string;
  // ActiveYN
  isActive: boolean;
  // 역할코드
  roleCd: RoleCode;
  // 사번
  employeeNum: string;
  // MADN
  madn: string;
  // HARDPHONE_SADN
  hardSadn: string;
  // FMC_SADN
  fmcSadn: string;
  // CALL_MODE
  callMode: CallMode;
  // 사용자명(영어)
  nameEn: string;
  // 사용자명(중국어)
  nameCn: string;
  // 직급(영어)
  gradeEn: string;
  // 직급(중국어)
  gradeCn: string;
  // 부서명(영어)
  deptNameEn: string;
  // 부서명(중국어)
  deptNameCn: string;
  // 부서SEQ
  deptSeq: number;
  // 이용약관동의여부YN
  isPrivacyAgree: boolean;
  // 유효접속여부YN
  isValidLogin: boolean;
  // 임직원유형(s)
  employeeType: EmployeeType;
}
export interface DeptUserData extends ProtocolStream {
  // DivCD(s)
  divCd: string;
  // {부서정보-S}...
  userInfos: UserInfoSS[];
}
export interface DeptUserResponse extends ProtocolResponse {
  // DivCD(s)
  divCd: string;
  // 전체갯수(n)
  pageTotalCount: number;
  // 요청리스트(n)
  pageListCount?: number;
  // 페이지(n)
  pageCurrent?: number;
}

export const encodeDeptUser: ProtocolEncoder<DeptUserRequest> = (
  req: DeptUserRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({ type: PacketBodyValue.String, value: req.divCd });
  bodyList.push({ type: PacketBodyValue.String, value: req.companyCode });
  bodyList.push({ type: PacketBodyValue.Integer, value: req.seq || 0 });
  bodyList.push({ type: PacketBodyValue.String, value: req.search || '' });
  bodyList.push({ type: PacketBodyValue.String, value: req.searchRange });
  bodyList.push({
    type: PacketBodyValue.Integer,
    value: req.pageListCount || 0
  });
  bodyList.push({ type: PacketBodyValue.Integer, value: req.pageCurrent || 0 });
  bodyList.push({ type: PacketBodyValue.String, value: req.senderCompanyCode });
  bodyList.push({
    type: PacketBodyValue.String,
    value: req.senderEmployeeType
  });

  return bodyList;
};

export const decodeDeptUserData: ProtocolDecoder<DeptUserData> = (
  message: ProtocolMessage
) => {
  const userInfos: UserInfoSS[] = [];
  message.bodyList.slice(1).forEach(userInfo => {
    const info = userInfo.split(BodyStringDivider);
    userInfos.push({
      seq: Number(info[0]),
      name: info[1],
      profileImageFile: info[2],
      grade: info[3],
      intro: info[4],
      companyCode: info[5],
      hpNumber: info[6],
      lineNumber: info[7],
      email: info[8],
      isMobile: info[9] === 'Y' ? true : false,
      deptName: info[10],
      order: info[11],
      isActive: info[12] === 'Y' ? true : false,
      roleCd: info[13] as RoleCode,
      employeeNum: info[14],
      madn: info[15],
      hardSadn: info[16],
      fmcSadn: info[17],
      callMode: info[18] as CallMode,
      nameEn: info[19],
      nameCn: info[20],
      gradeEn: info[21],
      gradeCn: info[22],
      deptNameEn: info[23],
      deptNameCn: info[24],
      deptSeq: info[25],
      isPrivacyAgree: info[26] === 'Y' ? true : false,
      isValidLogin: info[27] === 'Y' ? true : false,
      employeeType: info[28] as EmployeeType
    });
  });
  return {
    divCd: message.bodyList[0],
    userInfos
  } as DeptUserData;
};

export const decodeDeptUser: ProtocolDecoder<DeptUserResponse> = (
  message: ProtocolMessage
) => {
  return {
    divCd: message.bodyList[0],
    pageTotalCount: message.bodyList[1],
    pageListCount: message.bodyList[2],
    pageCurrent: message.bodyList[3]
  } as DeptUserResponse;
};
